@@ -22,6 +22,7 @@ from subprocess import TimeoutExpired
 import re
 import contextlib
 from collections.abc import Iterable
+from collections.abc import Sequence
 
 import scipy as sp
 from functools import wraps
@@ -34,11 +35,11 @@ from unittest import TestCase
 
 # WindowsError only exist on Windows
 try:
-    WindowsError
+    WindowsError  # type: ignore
 except NameError:
     WindowsError = None
 
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose as np_assert_allclose
 from numpy.testing import assert_almost_equal
 from numpy.testing import assert_approx_equal
 from numpy.testing import assert_array_equal
@@ -48,24 +49,34 @@ import numpy as np
 import joblib
 
 import sklearn
-from sklearn.utils import IS_PYPY, _IS_32BIT
+from sklearn.utils import (
+    IS_PYPY,
+    _IS_32BIT,
+    _in_unstable_openblas_configuration,
+)
 from sklearn.utils.multiclass import check_classification_targets
 from sklearn.utils.validation import (
     check_array,
     check_is_fitted,
     check_X_y,
 )
+from sklearn.utils.fixes import threadpool_info
 
 
-__all__ = ["assert_raises",
-           "assert_raises_regexp",
-           "assert_array_equal",
-           "assert_almost_equal",
-           "assert_array_almost_equal", "assert_array_less",
-           "assert_approx_equal", "assert_allclose",
-           "assert_run_python_script", "SkipTest"]
+__all__ = [
+    "assert_raises",
+    "assert_raises_regexp",
+    "assert_array_equal",
+    "assert_almost_equal",
+    "assert_array_almost_equal",
+    "assert_array_less",
+    "assert_approx_equal",
+    "assert_allclose",
+    "assert_run_python_script",
+    "SkipTest",
+]
 
-_dummy = TestCase('__init__')
+_dummy = TestCase("__init__")
 assert_raises = _dummy.assertRaises
 SkipTest = unittest.case.SkipTest
 assert_dict_equal = _dummy.assertDictEqual
@@ -77,140 +88,6 @@ assert_raises_regex = _dummy.assertRaisesRegex
 assert_raises_regexp = assert_raises_regex
 
 
-def assert_warns(warning_class, func, *args, **kw):
-    """Test that a certain warning occurs.
-
-    Parameters
-    ----------
-    warning_class : the warning class
-        The class to test for, e.g. UserWarning.
-
-    func : callable
-        Callable object to trigger warnings.
-
-    *args : the positional arguments to `func`.
-
-    **kw : the keyword arguments to `func`
-
-    Returns
-    -------
-    result : the return value of `func`
-
-    """
-    with warnings.catch_warnings(record=True) as w:
-        # Cause all warnings to always be triggered.
-        warnings.simplefilter("always")
-        # Trigger a warning.
-        result = func(*args, **kw)
-        if hasattr(np, 'FutureWarning'):
-            # Filter out numpy-specific warnings in numpy >= 1.9
-            w = [e for e in w
-                 if e.category is not np.VisibleDeprecationWarning]
-
-        # Verify some things
-        if not len(w) > 0:
-            raise AssertionError("No warning raised when calling %s"
-                                 % func.__name__)
-
-        found = any(warning.category is warning_class for warning in w)
-        if not found:
-            raise AssertionError("%s did not give warning: %s( is %s)"
-                                 % (func.__name__, warning_class, w))
-    return result
-
-
-def assert_warns_message(warning_class, message, func, *args, **kw):
-    # very important to avoid uncontrolled state propagation
-    """Test that a certain warning occurs and with a certain message.
-
-    Parameters
-    ----------
-    warning_class : the warning class
-        The class to test for, e.g. UserWarning.
-
-    message : str or callable
-        The message or a substring of the message to test for. If callable,
-        it takes a string as the argument and will trigger an AssertionError
-        if the callable returns `False`.
-
-    func : callable
-        Callable object to trigger warnings.
-
-    *args : the positional arguments to `func`.
-
-    **kw : the keyword arguments to `func`.
-
-    Returns
-    -------
-    result : the return value of `func`
-
-    """
-    with warnings.catch_warnings(record=True) as w:
-        # Cause all warnings to always be triggered.
-        warnings.simplefilter("always")
-        if hasattr(np, 'FutureWarning'):
-            # Let's not catch the numpy internal DeprecationWarnings
-            warnings.simplefilter('ignore', np.VisibleDeprecationWarning)
-        # Trigger a warning.
-        result = func(*args, **kw)
-        # Verify some things
-        if not len(w) > 0:
-            raise AssertionError("No warning raised when calling %s"
-                                 % func.__name__)
-
-        found = [issubclass(warning.category, warning_class) for warning in w]
-        if not any(found):
-            raise AssertionError("No warning raised for %s with class "
-                                 "%s"
-                                 % (func.__name__, warning_class))
-
-        message_found = False
-        # Checks the message of all warnings belong to warning_class
-        for index in [i for i, x in enumerate(found) if x]:
-            # substring will match, the entire message with typo won't
-            msg = w[index].message  # For Python 3 compatibility
-            msg = str(msg.args[0] if hasattr(msg, 'args') else msg)
-            if callable(message):  # add support for certain tests
-                check_in_message = message
-            else:
-                def check_in_message(msg): return message in msg
-
-            if check_in_message(msg):
-                message_found = True
-                break
-
-        if not message_found:
-            raise AssertionError("Did not receive the message you expected "
-                                 "('%s') for <%s>, got: '%s'"
-                                 % (message, func.__name__, msg))
-
-    return result
-
-
-def assert_warns_div0(func, *args, **kw):
-    """Assume that numpy's warning for divide by zero is raised.
-
-    Handles the case of platforms that do not support warning on divide by
-    zero.
-
-    Parameters
-    ----------
-    func
-    *args
-    **kw
-    """
-
-    with np.errstate(divide='warn', invalid='warn'):
-        try:
-            assert_warns(RuntimeWarning, np.divide, 1, np.zeros(1))
-        except AssertionError:
-            # This platform does not report numpy divide by zeros
-            return func(*args, **kw)
-        return assert_warns_message(RuntimeWarning,
-                                    'invalid value encountered',
-                                    func, *args, **kw)
-
-
 # To remove when we support numpy 1.7
 def assert_no_warnings(func, *args, **kw):
     """
@@ -222,18 +99,18 @@ def assert_no_warnings(func, *args, **kw):
     """
     # very important to avoid uncontrolled state propagation
     with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter('always')
+        warnings.simplefilter("always")
 
         result = func(*args, **kw)
-        if hasattr(np, 'FutureWarning'):
+        if hasattr(np, "FutureWarning"):
             # Filter out numpy-specific warnings in numpy >= 1.9
-            w = [e for e in w
-                 if e.category is not np.VisibleDeprecationWarning]
+            w = [e for e in w if e.category is not np.VisibleDeprecationWarning]
 
         if len(w) > 0:
-            raise AssertionError("Got warnings when calling %s: [%s]"
-                                 % (func.__name__,
-                                    ', '.join(str(warning) for warning in w)))
+            raise AssertionError(
+                "Got warnings when calling %s: [%s]"
+                % (func.__name__, ", ".join(str(warning) for warning in w))
+            )
     return result
 
 
@@ -253,6 +130,8 @@ def ignore_warnings(obj=None, category=Warning):
 
     Examples
     --------
+    >>> import warnings
+    >>> from sklearn.utils._testing import ignore_warnings
     >>> with ignore_warnings():
     ...     warnings.warn('buhuhuhu')
 
@@ -271,8 +150,8 @@ def ignore_warnings(obj=None, category=Warning):
             "'obj' should be a callable where you want to ignore warnings. "
             "You passed a warning class instead: 'obj={warning_name}'. "
             "If you want to pass a warning class to ignore_warnings, "
-            "you should use 'category={warning_name}'".format(
-                warning_name=warning_name))
+            "you should use 'category={warning_name}'".format(warning_name=warning_name)
+        )
     elif callable(obj):
         return _IgnoreWarnings(category=category)(obj)
     else:
@@ -294,13 +173,14 @@ class _IgnoreWarnings:
 
     def __init__(self, category):
         self._record = True
-        self._module = sys.modules['warnings']
+        self._module = sys.modules["warnings"]
         self._entered = False
         self.log = []
         self.category = category
 
     def __call__(self, fn):
         """Decorator to catch and hide warnings without visual nesting."""
+
         @wraps(fn)
         def wrapper(*args, **kwargs):
             with warnings.catch_warnings():
@@ -313,7 +193,7 @@ class _IgnoreWarnings:
         args = []
         if self._record:
             args.append("record=True")
-        if self._module is not sys.modules['warnings']:
+        if self._module is not sys.modules["warnings"]:
             args.append("module=%r" % self._module)
         name = type(self).__name__
         return "%s(%s)" % (name, ", ".join(args))
@@ -363,9 +243,10 @@ def assert_raise_message(exceptions, message, function, *args, **kwargs):
     except exceptions as e:
         error_message = str(e)
         if message not in error_message:
-            raise AssertionError("Error message does not include the expected"
-                                 " string: %r. Observed error message: %r" %
-                                 (message, error_message))
+            raise AssertionError(
+                "Error message does not include the expected"
+                " string: %r. Observed error message: %r" % (message, error_message)
+            )
     else:
         # concatenate exception names
         if isinstance(exceptions, tuple):
@@ -373,11 +254,83 @@ def assert_raise_message(exceptions, message, function, *args, **kwargs):
         else:
             names = exceptions.__name__
 
-        raise AssertionError("%s not raised by %s" %
-                             (names, function.__name__))
+        raise AssertionError("%s not raised by %s" % (names, function.__name__))
 
 
-def assert_allclose_dense_sparse(x, y, rtol=1e-07, atol=1e-9, err_msg=''):
+def assert_allclose(
+    actual, desired, rtol=None, atol=0.0, equal_nan=True, err_msg="", verbose=True
+):
+    """dtype-aware variant of numpy.testing.assert_allclose
+
+    This variant introspects the least precise floating point dtype
+    in the input argument and automatically sets the relative tolerance
+    parameter to 1e-4 float32 and use 1e-7 otherwise (typically float64
+    in scikit-learn).
+
+    `atol` is always left to 0. by default. It should be adjusted manually
+    to an assertion-specific value in case there are null values expected
+    in `desired`.
+
+    The aggregate tolerance is `atol + rtol * abs(desired)`.
+
+    Parameters
+    ----------
+    actual : array_like
+        Array obtained.
+    desired : array_like
+        Array desired.
+    rtol : float, optional, default=None
+        Relative tolerance.
+        If None, it is set based on the provided arrays' dtypes.
+    atol : float, optional, default=0.
+        Absolute tolerance.
+    equal_nan : bool, optional, default=True
+        If True, NaNs will compare equal.
+    err_msg : str, optional, default=''
+        The error message to be printed in case of failure.
+    verbose : bool, optional, default=True
+        If True, the conflicting values are appended to the error message.
+
+    Raises
+    ------
+    AssertionError
+        If actual and desired are not equal up to specified precision.
+
+    See Also
+    --------
+    numpy.testing.assert_allclose
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.utils._testing import assert_allclose
+    >>> x = [1e-5, 1e-3, 1e-1]
+    >>> y = np.arccos(np.cos(x))
+    >>> assert_allclose(x, y, rtol=1e-5, atol=0)
+    >>> a = np.full(shape=10, fill_value=1e-5, dtype=np.float32)
+    >>> assert_allclose(a, 1e-5)
+    """
+    dtypes = []
+
+    actual, desired = np.asanyarray(actual), np.asanyarray(desired)
+    dtypes = [actual.dtype, desired.dtype]
+
+    if rtol is None:
+        rtols = [1e-4 if dtype == np.float32 else 1e-7 for dtype in dtypes]
+        rtol = max(rtols)
+
+    np_assert_allclose(
+        actual,
+        desired,
+        rtol=rtol,
+        atol=atol,
+        equal_nan=equal_nan,
+        err_msg=err_msg,
+        verbose=verbose,
+    )
+
+
+def assert_allclose_dense_sparse(x, y, rtol=1e-07, atol=1e-9, err_msg=""):
     """Assert allclose for sparse and dense data.
 
     Both x and y need to be either sparse or dense, they
@@ -414,8 +367,9 @@ def assert_allclose_dense_sparse(x, y, rtol=1e-07, atol=1e-9, err_msg=''):
         # both dense
         assert_allclose(x, y, rtol=rtol, atol=atol, err_msg=err_msg)
     else:
-        raise ValueError("Can only compare two sparse matrices,"
-                         " not a sparse matrix and an array.")
+        raise ValueError(
+            "Can only compare two sparse matrices, not a sparse matrix and an array."
+        )
 
 
 def set_random_state(estimator, random_state=0):
@@ -437,14 +391,18 @@ def set_random_state(estimator, random_state=0):
 try:
     import pytest
 
-    skip_if_32bit = pytest.mark.skipif(_IS_32BIT,
-                                       reason='skipped on 32bit platforms')
-    skip_travis = pytest.mark.skipif(os.environ.get('TRAVIS') == 'true',
-                                     reason='skip on travis')
-    fails_if_pypy = pytest.mark.xfail(IS_PYPY,
-                                      reason='not compatible with PyPy')
-    skip_if_no_parallel = pytest.mark.skipif(not joblib.parallel.mp,
-                                             reason="joblib is in serial mode")
+    skip_if_32bit = pytest.mark.skipif(_IS_32BIT, reason="skipped on 32bit platforms")
+    skip_travis = pytest.mark.skipif(
+        os.environ.get("TRAVIS") == "true", reason="skip on travis"
+    )
+    fails_if_pypy = pytest.mark.xfail(IS_PYPY, reason="not compatible with PyPy")
+    fails_if_unstable_openblas = pytest.mark.xfail(
+        _in_unstable_openblas_configuration(),
+        reason="OpenBLAS is unstable for this configuration",
+    )
+    skip_if_no_parallel = pytest.mark.skipif(
+        not joblib.parallel.mp, reason="joblib is in serial mode"
+    )
 
     #  Decorator for tests involving both BLAS calls and multiprocessing.
     #
@@ -466,14 +424,14 @@ try:
     #  default.
 
     if_safe_multiprocessing_with_blas = pytest.mark.skipif(
-            sys.platform == 'darwin',
-            reason="Possible multi-process bug with some BLAS")
+        sys.platform == "darwin", reason="Possible multi-process bug with some BLAS"
+    )
 except ImportError:
     pass
 
 
 def check_skip_network():
-    if int(os.environ.get('SKLEARN_SKIP_NETWORK_TESTS', 0)):
+    if int(os.environ.get("SKLEARN_SKIP_NETWORK_TESTS", 0)):
         raise SkipTest("Text tutorial requires large dataset download")
 
 
@@ -499,34 +457,91 @@ class TempMemmap:
     data
     mmap_mode : str, default='r'
     """
-    def __init__(self, data, mmap_mode='r'):
+
+    def __init__(self, data, mmap_mode="r"):
         self.mmap_mode = mmap_mode
         self.data = data
 
     def __enter__(self):
         data_read_only, self.temp_folder = create_memmap_backed_data(
-            self.data, mmap_mode=self.mmap_mode, return_folder=True)
+            self.data, mmap_mode=self.mmap_mode, return_folder=True
+        )
         return data_read_only
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         _delete_folder(self.temp_folder)
 
 
-def create_memmap_backed_data(data, mmap_mode='r', return_folder=False):
+def _create_memmap_backed_array(array, filename, mmap_mode):
+    # https://numpy.org/doc/stable/reference/generated/numpy.memmap.html
+    fp = np.memmap(filename, dtype=array.dtype, mode="w+", shape=array.shape)
+    fp[:] = array[:]  # write array to memmap array
+    fp.flush()
+    memmap_backed_array = np.memmap(
+        filename, dtype=array.dtype, mode=mmap_mode, shape=array.shape
+    )
+    return memmap_backed_array
+
+
+def _create_aligned_memmap_backed_arrays(data, mmap_mode, folder):
+    if isinstance(data, np.ndarray):
+        filename = op.join(folder, "data.dat")
+        return _create_memmap_backed_array(data, filename, mmap_mode)
+
+    if isinstance(data, Sequence) and all(
+        isinstance(each, np.ndarray) for each in data
+    ):
+        return [
+            _create_memmap_backed_array(
+                array, op.join(folder, f"data{index}.dat"), mmap_mode
+            )
+            for index, array in enumerate(data)
+        ]
+
+    raise ValueError(
+        "When creating aligned memmap-backed arrays, input must be a single array or a"
+        " sequence of arrays"
+    )
+
+
+def create_memmap_backed_data(data, mmap_mode="r", return_folder=False, aligned=False):
     """
     Parameters
     ----------
     data
     mmap_mode : str, default='r'
     return_folder :  bool, default=False
+    aligned : bool, default=False
+        If True, if input is a single numpy array and if the input array is aligned,
+        the memory mapped array will also be aligned. This is a workaround for
+        https://github.com/joblib/joblib/issues/563.
     """
-    temp_folder = tempfile.mkdtemp(prefix='sklearn_testing_')
+    temp_folder = tempfile.mkdtemp(prefix="sklearn_testing_")
     atexit.register(functools.partial(_delete_folder, temp_folder, warn=True))
-    filename = op.join(temp_folder, 'data.pkl')
-    joblib.dump(data, filename)
-    memmap_backed_data = joblib.load(filename, mmap_mode=mmap_mode)
-    result = (memmap_backed_data if not return_folder
-              else (memmap_backed_data, temp_folder))
+    # OpenBLAS is known to segfault with unaligned data on the Prescott
+    # architecture so force aligned=True on Prescott. For more details, see:
+    # https://github.com/scipy/scipy/issues/14886
+    has_prescott_openblas = any(
+        True
+        for info in threadpool_info()
+        if info["internal_api"] == "openblas"
+        # Prudently assume Prescott might be the architecture if it is unknown.
+        and info.get("architecture", "prescott").lower() == "prescott"
+    )
+    if has_prescott_openblas:
+        aligned = True
+
+    if aligned:
+        memmap_backed_data = _create_aligned_memmap_backed_arrays(
+            data, mmap_mode, temp_folder
+        )
+    else:
+        filename = op.join(temp_folder, "data.pkl")
+        joblib.dump(data, filename)
+        memmap_backed_data = joblib.load(filename, mmap_mode=mmap_mode)
+    result = (
+        memmap_backed_data if not return_folder else (memmap_backed_data, temp_folder)
+    )
     return result
 
 
@@ -541,11 +556,17 @@ def _get_args(function, varargs=False):
     except ValueError:
         # Error on builtin C function
         return []
-    args = [key for key, param in params.items()
-            if param.kind not in (param.VAR_POSITIONAL, param.VAR_KEYWORD)]
+    args = [
+        key
+        for key, param in params.items()
+        if param.kind not in (param.VAR_POSITIONAL, param.VAR_KEYWORD)
+    ]
     if varargs:
-        varargs = [param.name for param in params.values()
-                   if param.kind == param.VAR_POSITIONAL]
+        varargs = [
+            param.name
+            for param in params.values()
+            if param.kind == param.VAR_POSITIONAL
+        ]
         if len(varargs) == 0:
             varargs = None
         return args, varargs
@@ -573,10 +594,10 @@ def _get_func_name(func):
 
     qualname = func.__qualname__
     if qualname != func.__name__:
-        parts.append(qualname[:qualname.find('.')])
+        parts.append(qualname[: qualname.find(".")])
 
     parts.append(func.__name__)
-    return '.'.join(parts)
+    return ".".join(parts)
 
 
 def check_docstring_parameters(func, doc=None, ignore=None):
@@ -597,56 +618,72 @@ def check_docstring_parameters(func, doc=None, ignore=None):
         A list of string describing the incorrect results.
     """
     from numpydoc import docscrape
+
     incorrect = []
     ignore = [] if ignore is None else ignore
 
     func_name = _get_func_name(func)
-    if (not func_name.startswith('sklearn.') or
-            func_name.startswith('sklearn.externals')):
+    if not func_name.startswith("sklearn.") or func_name.startswith(
+        "sklearn.externals"
+    ):
         return incorrect
     # Don't check docstring for property-functions
     if inspect.isdatadescriptor(func):
         return incorrect
     # Don't check docstring for setup / teardown pytest functions
-    if func_name.split('.')[-1] in ('setup_module', 'teardown_module'):
+    if func_name.split(".")[-1] in ("setup_module", "teardown_module"):
         return incorrect
     # Dont check estimator_checks module
-    if func_name.split('.')[2] == 'estimator_checks':
+    if func_name.split(".")[2] == "estimator_checks":
         return incorrect
     # Get the arguments from the function signature
     param_signature = list(filter(lambda x: x not in ignore, _get_args(func)))
     # drop self
-    if len(param_signature) > 0 and param_signature[0] == 'self':
-        param_signature.remove('self')
+    if len(param_signature) > 0 and param_signature[0] == "self":
+        param_signature.remove("self")
 
     # Analyze function's docstring
     if doc is None:
-        with warnings.catch_warnings(record=True) as w:
+        records = []
+        with warnings.catch_warnings(record=True):
+            warnings.simplefilter("error", UserWarning)
             try:
                 doc = docscrape.FunctionDoc(func)
+            except UserWarning as exp:
+                if "potentially wrong underline length" in str(exp):
+                    # Catch warning raised as of numpydoc 1.2 when
+                    # the underline length for a section of a docstring
+                    # is not consistent.
+                    message = str(exp).split("\n")[:3]
+                    incorrect += [f"In function: {func_name}"] + message
+                    return incorrect
+                records.append(str(exp))
             except Exception as exp:
-                incorrect += [func_name + ' parsing error: ' + str(exp)]
+                incorrect += [func_name + " parsing error: " + str(exp)]
                 return incorrect
-        if len(w):
-            raise RuntimeError('Error for %s:\n%s' % (func_name, w[0]))
+        if len(records):
+            raise RuntimeError("Error for %s:\n%s" % (func_name, records[0]))
 
     param_docs = []
-    for name, type_definition, param_doc in doc['Parameters']:
+    for name, type_definition, param_doc in doc["Parameters"]:
         # Type hints are empty only if parameter name ended with :
         if not type_definition.strip():
-            if ':' in name and name[:name.index(':')][-1:].strip():
-                incorrect += [func_name +
-                              ' There was no space between the param name and '
-                              'colon (%r)' % name]
-            elif name.rstrip().endswith(':'):
-                incorrect += [func_name +
-                              ' Parameter %r has an empty type spec. '
-                              'Remove the colon' % (name.lstrip())]
+            if ":" in name and name[: name.index(":")][-1:].strip():
+                incorrect += [
+                    func_name
+                    + " There was no space between the param name and colon (%r)" % name
+                ]
+            elif name.rstrip().endswith(":"):
+                incorrect += [
+                    func_name
+                    + " Parameter %r has an empty type spec. Remove the colon"
+                    % (name.lstrip())
+                ]
 
         # Create a list of parameters to compare with the parameters gotten
         # from the func signature
-        if '*' not in name:
-            param_docs.append(name.split(':')[0].strip('` '))
+        if "*" not in name:
+            param_docs.append(name.split(":")[0].strip("` "))
 
     # If one of the docstring's parameters had an error then return that
     # incorrect message
@@ -663,20 +700,25 @@ def check_docstring_parameters(func, doc=None, ignore=None):
     message = []
     for i in range(min(len(param_docs), len(param_signature))):
         if param_signature[i] != param_docs[i]:
-            message += ["There's a parameter name mismatch in function"
-                        " docstring w.r.t. function signature, at index %s"
-                        " diff: %r != %r" %
-                        (i, param_signature[i], param_docs[i])]
+            message += [
+                "There's a parameter name mismatch in function"
+                " docstring w.r.t. function signature, at index %s"
+                " diff: %r != %r" % (i, param_signature[i], param_docs[i])
+            ]
             break
     if len(param_signature) > len(param_docs):
-        message += ["Parameters in function docstring have less items w.r.t."
-                    " function signature, first missing item: %s" %
-                    param_signature[len(param_docs)]]
+        message += [
+            "Parameters in function docstring have less items w.r.t."
+            " function signature, first missing item: %s"
+            % param_signature[len(param_docs)]
+        ]
 
     elif len(param_signature) < len(param_docs):
-        message += ["Parameters in function docstring have more items w.r.t."
-                    " function signature, first extra item: %s" %
-                    param_docs[len(param_signature)]]
+        message += [
+            "Parameters in function docstring have more items w.r.t."
+            " function signature, first extra item: %s"
+            % param_docs[len(param_signature)]
+        ]
 
     # If there wasn't any difference in the parameters themselves between
     # docstring and signature including having the same length then return
@@ -693,14 +735,14 @@ def check_docstring_parameters(func, doc=None, ignore=None):
     message += ["Full diff:"]
 
     message.extend(
-        line.strip() for line in difflib.ndiff(param_signature_formatted,
-                                               param_docs_formatted)
+        line.strip()
+        for line in difflib.ndiff(param_signature_formatted, param_docs_formatted)
     )
 
     incorrect.extend(message)
 
     # Prepend function name
-    incorrect = ['In function: ' + func_name] + incorrect
+    incorrect = ["In function: " + func_name] + incorrect
 
     return incorrect
 
@@ -720,68 +762,92 @@ def assert_run_python_script(source_code, timeout=60):
     timeout : int, default=60
         Time in seconds before timeout.
     """
-    fd, source_file = tempfile.mkstemp(suffix='_src_test_sklearn.py')
+    fd, source_file = tempfile.mkstemp(suffix="_src_test_sklearn.py")
     os.close(fd)
     try:
-        with open(source_file, 'wb') as f:
-            f.write(source_code.encode('utf-8'))
+        with open(source_file, "wb") as f:
+            f.write(source_code.encode("utf-8"))
         cmd = [sys.executable, source_file]
-        cwd = op.normpath(op.join(op.dirname(sklearn.__file__), '..'))
+        cwd = op.normpath(op.join(op.dirname(sklearn.__file__), ".."))
         env = os.environ.copy()
         try:
             env["PYTHONPATH"] = os.pathsep.join([cwd, env["PYTHONPATH"]])
         except KeyError:
             env["PYTHONPATH"] = cwd
-        kwargs = {
-            'cwd': cwd,
-            'stderr': STDOUT,
-            'env': env
-        }
+        kwargs = {"cwd": cwd, "stderr": STDOUT, "env": env}
         # If coverage is running, pass the config file to the subprocess
         coverage_rc = os.environ.get("COVERAGE_PROCESS_START")
         if coverage_rc:
-            kwargs['env']['COVERAGE_PROCESS_START'] = coverage_rc
+            kwargs["env"]["COVERAGE_PROCESS_START"] = coverage_rc
 
-        kwargs['timeout'] = timeout
+        kwargs["timeout"] = timeout
         try:
             try:
                 out = check_output(cmd, **kwargs)
             except CalledProcessError as e:
-                raise RuntimeError(u"script errored with output:\n%s"
-                                   % e.output.decode('utf-8'))
+                raise RuntimeError(
+                    "script errored with output:\n%s" % e.output.decode("utf-8")
+                )
             if out != b"":
-                raise AssertionError(out.decode('utf-8'))
+                raise AssertionError(out.decode("utf-8"))
         except TimeoutExpired as e:
-            raise RuntimeError(u"script timeout, output so far:\n%s"
-                               % e.output.decode('utf-8'))
+            raise RuntimeError(
+                "script timeout, output so far:\n%s" % e.output.decode("utf-8")
+            )
     finally:
         os.unlink(source_file)
 
 
-def _convert_container(container, constructor_name, columns_name=None):
-    if constructor_name == 'list':
-        return list(container)
-    elif constructor_name == 'tuple':
-        return tuple(container)
-    elif constructor_name == 'array':
-        return np.asarray(container)
-    elif constructor_name == 'sparse':
-        return sp.sparse.csr_matrix(container)
-    elif constructor_name == 'dataframe':
-        pd = pytest.importorskip('pandas')
-        return pd.DataFrame(container, columns=columns_name)
-    elif constructor_name == 'series':
-        pd = pytest.importorskip('pandas')
-        return pd.Series(container)
-    elif constructor_name == 'index':
-        pd = pytest.importorskip('pandas')
-        return pd.Index(container)
-    elif constructor_name == 'slice':
+def _convert_container(container, constructor_name, columns_name=None, dtype=None):
+    """Convert a given container to a specific array-like with a dtype.
+
+    Parameters
+    ----------
+    container : array-like
+        The container to convert.
+    constructor_name : {"list", "tuple", "array", "sparse", "dataframe", \
+            "series", "index", "slice", "sparse_csr", "sparse_csc"}
+        The type of the returned container.
+    columns_name : index or array-like, default=None
+        For pandas container supporting `columns_names`, it will affect
+        specific names.
+    dtype : dtype, default=None
+        Force the dtype of the container. Does not apply to `"slice"`
+        container.
+
+    Returns
+    -------
+    converted_container
+    """
+    if constructor_name == "list":
+        if dtype is None:
+            return list(container)
+        else:
+            return np.asarray(container, dtype=dtype).tolist()
+    elif constructor_name == "tuple":
+        if dtype is None:
+            return tuple(container)
+        else:
+            return tuple(np.asarray(container, dtype=dtype).tolist())
+    elif constructor_name == "array":
+        return np.asarray(container, dtype=dtype)
+    elif constructor_name == "sparse":
+        return sp.sparse.csr_matrix(container, dtype=dtype)
+    elif constructor_name == "dataframe":
+        pd = pytest.importorskip("pandas")
+        return pd.DataFrame(container, columns=columns_name, dtype=dtype)
+    elif constructor_name == "series":
+        pd = pytest.importorskip("pandas")
+        return pd.Series(container, dtype=dtype)
+    elif constructor_name == "index":
+        pd = pytest.importorskip("pandas")
+        return pd.Index(container, dtype=dtype)
+    elif constructor_name == "slice":
         return slice(container[0], container[1])
-    elif constructor_name == 'sparse_csr':
-        return sp.sparse.csr_matrix(container)
-    elif constructor_name == 'sparse_csc':
-        return sp.sparse.csc_matrix(container)
+    elif constructor_name == "sparse_csr":
+        return sp.sparse.csr_matrix(container, dtype=dtype)
+    elif constructor_name == "sparse_csc":
+        return sp.sparse.csc_matrix(container, dtype=dtype)
 
 
 def raises(expected_exc_type, match=None, may_pass=False, err_msg=None):
@@ -845,9 +911,7 @@ class _Raises(contextlib.AbstractContextManager):
             if self.may_pass:
                 return True  # CM is happy
             else:
-                err_msg = (
-                    self.err_msg or f"Did not raise: {self.expected_exc_types}"
-                )
+                err_msg = self.err_msg or f"Did not raise: {self.expected_exc_types}"
                 raise AssertionError(err_msg)
 
         if not any(
@@ -862,12 +926,9 @@ class _Raises(contextlib.AbstractContextManager):
         if self.matches is not None:
             err_msg = self.err_msg or (
                 "The error message should contain one of the following "
-                "patterns:\n{}\nGot {}".format(
-                    "\n".join(self.matches), str(exc_value)
-                )
+                "patterns:\n{}\nGot {}".format("\n".join(self.matches), str(exc_value))
             )
-            if not any(re.search(match, str(exc_value))
-                       for match in self.matches):
+            if not any(re.search(match, str(exc_value)) for match in self.matches):
                 raise AssertionError(err_msg) from exc_value
             self.raised_and_matched = True
 
@@ -883,6 +944,7 @@ class MinimalClassifier:
     * within a `Pipeline` in `test_pipeline.py`;
     * within a `SearchCV` in `test_search.py`.
     """
+
     _estimator_type = "classifier"
 
     def __init__(self, param=None):
@@ -918,6 +980,7 @@ class MinimalClassifier:
 
     def score(self, X, y):
         from sklearn.metrics import accuracy_score
+
         return accuracy_score(y, self.predict(X))
 
 
@@ -930,6 +993,7 @@ class MinimalRegressor:
     * within a `Pipeline` in `test_pipeline.py`;
     * within a `SearchCV` in `test_search.py`.
     """
+
     _estimator_type = "regressor"
 
     def __init__(self, param=None):
@@ -956,6 +1020,7 @@ class MinimalRegressor:
 
     def score(self, X, y):
         from sklearn.metrics import r2_score
+
         return r2_score(y, self.predict(X))
 
 
@@ -982,7 +1047,7 @@ class MinimalTransformer:
         return self
 
     def fit(self, X, y=None):
-        X = check_array(X)
+        check_array(X)
         self.is_fitted_ = True
         return self
 
